package exec;

use strict;
use POSIX;

##############################################################################
# exec package - routines for handling child processes.  
#
# exports reap_children, fork_and_exec
#
# copyright (c) 1997 by Steve Romig and The Ohio State University
#
##############################################################################
# $Id: exec.pm,v 1.1 1997/05/27 21:03:47 romig Exp $
##############################################################################
# $Log: exec.pm,v $
# Revision 1.1  1997/05/27 21:03:47  romig
# Initial revision
#

#
# list of child process ids (keys, no values).
#
my %children = ();

#
# waitpids on each child to reap those that have died.  list of
# children process id's is in %children.  set as sigchld handler 
# if you plan to use fork_and_exec (below).
#
sub reap_children {
    shift;

    my $child_pid;
    foreach $child_pid (keys %children) {
	if (waitpid($child_pid, &POSIX::WNOHANG)) {
	    delete($children{$child_pid});
	    print "child $child_pid died\n" if $main::debug;
	}
    }
}

#
# fork and execute the given command.  command can be an array of words,
# which will be passed to exec.
#
sub fork_and_exec {
    my(@command) = @_;
    my $pid;

    unless ($pid = fork) {
	print join(' ', @command), "\n"
	    if $main::debug;

	exec @command;
	die "exec failed ($!)\n";
    }

    $children{$pid} = 1;
}

1;

