package host_cache;

##############################################################################
# host_cache package - routines for looking up host names by name or 
# address, but using a cache to improve performance and decrease 
# requests to name servers and such.
#
# exports &get_host_by_addr, $name2ip, $ip2name
#
# copyright (c) 1997 by Steve Romig and The Ohio State University
#
##############################################################################
# $Id: host_cache.pm,v 1.1 1997/05/27 21:03:47 romig Exp $
##############################################################################
# $Log: host_cache.pm,v $
# Revision 1.1  1997/05/27 21:03:47  romig
# Initial revision
#

# provides
#
#    &cache_get_host_by_addr($ip_address)
#
#	given an ip address, look up the corresponding name with 
#	gethostbyaddr, cache ip->name and name->ip in %ip2name and 
#	%name2ip and return the name.  If gethostbyaddr doesn't find 
#	a name, cache and return the ip address instead.
#
#    $name = $ip2name{$ip_address}
#
#	mapping of ip addresses to names found through calls to 
#	&cache_get_host_by_addr.  If the given ip address didn't have 
#	a name, $ip2name contains the ip address as the name.
#
#    $ip_address = $name2ip{$name}
#
#	ditto, but reverse.
#
# fixme:
#    should allow you to read hostnames from file(s)
#
#    might need something similar for get_host_by_name
#

sub get_host_by_addr {
    local($ipaddr) = @_;

    if (defined($ip2name{$ipaddr})) {
	# If the requested entry is cached, look that up and return it

        return($ip2name{$ipaddr});
    } else {
	# otherwise, look it up with gethostbyaddr, save the results and 
	# return the name found.

        $result = (gethostbyaddr(pack('C4', split('\.', $ipaddr)), 2))[0];
	if (defined($result) && $result ne '') {
	    $result =~ tr/A-Z/a-z/;			# lowercase it
	} else {
	    $result = $ipaddr;
	}

        $ip2name{$ipaddr} = $result;		# cache name, ip address
	$name2ip{$result} = $ipaddr;
        return($result);
    }
}

1;
