package log_window;

use strict;

##############################################################################
# log_window package - create a window containing a scrollable list of 
# log files, plus support routines
#
# exports create
#
# copyright (c) 1997 by Steve Romig and The Ohio State University
#
##############################################################################
# $Id: log_window.pm,v 1.1 1997/05/27 21:03:47 romig Exp $
##############################################################################
# $Log: log_window.pm,v $
# Revision 1.1  1997/05/27 21:03:47  romig
# Initial revision
#

# exports create

#
# Refresh the main window listing the log files
# Given a text box, it fills with the filename,
# each tagged to execute a sub on double-click.
#
sub redraw_files {
    my($box, @file_search_list) = @_;

    my $target;
    my $file_pattern;
    my $file;
    my @stat;
    my @file_list = ();
    my $i;

    @file_list = &main::get_files(@file_search_list);

    # make it writable, delete everything
    #
    $box->configure(-state => 'normal');
    $box->delete('1.0', 'end');

    @file_list = sort @file_list;
    for ($i = 0; $i <= $#file_list; $i++) {
	@stat = stat($file_list[$i]);

	my $stuffit = sprintf("%7dK  %s\n", 
			      ($stat[7]+0)/1024, $file_list[$i]);
	chomp($stuffit) if $i == $#file_list;

        $box->insert('end', $stuffit, ['go', "go-$file_list[$i]"]);
    }

    $box->configure(-height => &main::min($#file_list+1, 30));

    $box->configure(-state => 'disabled');
}

#
# create the main window for reviewing a bunch of log files: the log 
# listing window.
#
# this scrollable window contains a list of all of the named log
# files, has buttons that allow you to exit the program and refresh
# the log listing.
# 
# double click on a log file brings up the summary window for that log
# file. 
#
sub create {
    my(@file_search_list) = @_;
    my @files;

    $log_window::main = new MainWindow;

    $log_window::main->title("Log Review");

    # create this here for references
    #
    my $text = $log_window::main->Text(-relief => 'sunken', 
					     -background => 
				       		$main::background_color,
					     -foreground => 
				       		$main::foreground_color,
					     -width => 80,
					     -setgrid => 'yes');

    &window::define_tags($text, \&summary_window::create);

    # frame for the top row of buttons: exit, refresh
    #
    # +-------------------------
    # | Exit Refresh Reports
    # |
    #
    my $button_frame = $log_window::main->Frame(-relief => 'raised', -bd => 2);
    $button_frame->pack(-side => 'top', -fill => 'x');

      my $exit_button = 
	  $button_frame->Button(
	      -text => "Exit",
	      -command => ['destroy', $log_window::main]);
      $exit_button->pack(-side => 'left');

      my $refresh_button = 
	  $button_frame->Button(
	      -text => "Refresh",
	      -command => sub {&redraw_files($text, @file_search_list)});
      $refresh_button->pack(-side => 'left');
			      
      my $view_menu = $button_frame->Menubutton(-text => 'Reports', 
						-underline => 0);

      $view_menu->command(-label   => 'All Reports',
			  -command => sub {&report_window::create('all', &main::get_files(@file_search_list))});

      $view_menu->command(-label   => 'Security Reports',
			  -command => sub {&report_window::create('security', &main::get_files(@file_search_list))});

      $view_menu->command(-label   => 'Summary Reports',
			  -command => sub {&report_window::create('summary', &main::get_files(@file_search_list))});

      $view_menu->pack(-side => 'left');

#      my $reports_button = 
#	  $button_frame->Button(
#	      -text => "Reports",
#	      -command => 
#                 sub {&report_window::create(&main::get_files(@file_search_list))});
#      $reports_button->pack(-side => 'left');
			      
    # field description for the text box
    #
    # +-------------------------
    # | Exit Refresh
    # | <description>
    # |
    #
    my $description = $log_window::main->Text(-relief => 'flat',
					      -width => 80,
					      -height => 1);
    $description->insert('0.0', '    Size  Name');
    $description->pack(-side => 'top',
		       -fill => 'x',
		       -expand => 'y');
    $description->configure(-state => 'disabled');

    # scrollable text box for the list of file names
    #
    # +-------------------------
    # | Exit Refresh
    # | <description>
    # |+------------------+ S
    # || list of logs...  | C
    # ||                  | R
    # ||                  | O
    # ||                  | L
    # ||                  | L
    # ++------------------+-----
    #
    # fill the text window
    &redraw_files($text, @file_search_list);

    $text->pack(-side => 'left',
		      -fill => 'both',
		      -expand => 'yes');

    # scroll bar for the text widget
    my $browse_scroll = 
	$log_window::main->Scrollbar(-command => ['yview', $text]);

    $text->configure(-yscrollcommand => ['set', $browse_scroll]);

    $browse_scroll->pack(-side => 'right', 
			 -fill => 'y');

    my @file_list = &main::get_files(@file_search_list);

    if ($#file_list == 0) {
        &summary_window::create($file_list[0]);
    }

    Tk::MainLoop;
}

1;
