package report;

use strict;
use FileHandle;

##############################################################################
# report package - routines to create and save various reports
#
# exports &make_and_save
#
# copyright (c) 1997 by Steve Romig and The Ohio State University
#
##############################################################################
# $Id$
##############################################################################
# $Log$
#

# sigh - for each report that we might generate, ought to list
# resources that are needed to create the report, then when it comes
# time to do the work, we can:
#
# for each log file
#     make list of resources needed
#     for each resource (since these will be gen'd on the fly
#	for each report
#	  do whatever
#
# which would be most efficient
#
# i think :-)
#
# for resources, i think there are 3:
# 
# - some of the reports can be generated from the summaries (e.g, nfs
#   traffic, portmon traffic
#
# - most of the reports can be generated from just searching the
#   contents of all of packets, one by one (e.g, tcpdump -q -x -r with
#   no selection criteria, piped to cleanup -i, then deal with each on
#   a case by case basis, as in "this is web c->s, grab the url", or
#   "this is something not web, look for crack output"
#
# - some possible searches will require the ability to both reassemble
#   the tcp sessions and to process backspace, erase and other common
#   editing things (e.g., to search in what they typed for names of
#   common cracking tools)
#
# - i suppose you could add the ability to interpret escape crap from
#   tty sessions and rebuild a picture of the ascii text seen, or to
#   rebuild what was seen in an x display.  proof left to the reader
#   :-) 

# ping, syn flood
# ping w/ non-0 data
# unknown services?

%report::info = (
    'crack' => {
	'title' => 	'Crack Logs',
	'extension' =>	'crack',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'sniff' => {
	'title' => 	'Sniffer Logs',
	'extension' =>	'sniff',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'passwd' => {
	'title' => 	'UNIX Password Files',
	'extension' =>	'passwd',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'web' => {
	'title' => 	'Web Visits',
	'extension' =>	'web',
	'type' =>	'summary',
	'uses-raw' =>	1,
    },

    'phf' => {
	'title' => 	'Possible cgi/phf attacks',
	'extension' =>	'phf',
	'type' =>	'security', 
	'uses-raw' =>	1,
   },

    'ftp' => {
	'title' => 	'FTP Summary',
	'extension' =>	'ftp',
	'type' =>	'summary',
	'uses-raw' =>	1,
    },

    'dcc-send' => {
	'title' => 	'IRC DCC Send/Receive Summary',
	'extension' =>	'dcc-send',
	'type' =>	'summary',
	'uses-raw' =>	1,
    },

    'irc-nick' => {
	'title' => 	'IRC NICKs Used',
	'extension' =>	'irc-nick',
	'type' =>	'summary',
	'uses-raw' =>	1,
    },

    'mail-sent' => {
	'title' => 	'Mail Sent Summary',
	'extension' =>	'mail-sent',
	'type' =>	'summary',
	'uses-raw' =>	1,
    },

    'mail-read' => {
	'title' => 	'Mail Read Summary',
	'extension' =>	'mail-read',
	'type' =>	'summary',
	'uses-raw' =>	1,
    },

    'hostprobe' => {
	'title' => 	'Host Probes',
	'extension' =>	'hostprobe',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'portprobe' => {
	'title' => 	'Port Probes',
	'extension' =>	'portprobe',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'synflood' => {
	'title' => 	'SYN Flood',
	'extension' =>	'synflood',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'pingflood' => {
	'title' => 	'PING Flood',
	'extension' =>	'pingflood',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'sendmail' => {
	'title' => 	'Sendmail debug, wiz, decode, pipe...',
	'extension' =>	'sendmail',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'ftpd' => {
	'title' => 	'Possible FTP Attacks',
	'extension' =>	'ftpd',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'rhosts' => {
	'title' => 	'References to .rhosts files',
	'extension' =>	'rhosts',
	'type' =>	'security',
	'uses-raw' =>	1,
    },

    'passphrase' => {
	'title' => 	'Possible PGP or SSH Pass Phrases',
	'extension' =>	'passphrase',
	'type' =>	'security',
	'uses-raw' =>	1,
    },
);

@report::security_list = ();
@report::summary_list = ();
@report::other_list = ();

foreach $report::tmp (keys %report::info) {
    my $type;
    if (defined($report::info{$report::tmp}{'type'})) {
	$type = $report::info{$report::tmp}{'type'};
    } else {
	$type = '';
    }

    if ($type eq 'security') {
	push(@report::security_list, $report::tmp);
    } elsif ($type eq 'summary') {
	push(@report::summary_list, $report::tmp);
    } else {
	push(@report::other_list, $report::tmp);	    
    }
}

@report::security_list = sort @report::security_list;
@report::summary_list = sort @report::summary_list;
@report::other_list = sort @report::other_list;

# split_and_check splits a list of report types and checks them for 
# validity, returning the list of reports found.  'all' is expanded to 
# the list of all known reports.  duplicates are collapsed.

sub split_and_check {
    my($list) = @_;
    my $rover;
    my $thing;
    my %result = ();
    my $error = 0;

    foreach $rover (split(/,/, $list)) {
	if (defined($report::info{$rover})) {
	    $result{$rover} = 1;
	} elsif ($rover eq 'all') {
	    foreach $thing (keys %report::info) {
		$result{$thing} = 1;
	    }
	} elsif ($rover eq 'security') {
	    foreach $thing (keys %report::info) {
		if ($report::info{$thing}{'type'} eq 'security') {
		    $result{$thing} = 1;
		}
	    }
	} elsif ($rover eq 'summary') {
	    foreach $thing (keys %report::info) {
		if ($report::info{$thing}{'type'} ne 'security') {
		    $result{$thing} = 1;
		}
	    }
	} else {
	    warn "there is no report named '$rover'.  known reports: ", 
	         join(' ', keys %report::info), "\n";
	    $error = 1;
	}
    }

    if (! $error) {
	return(keys %result);
    } else {
	return();
    }
}

sub make_and_save {
    my($file, $force, @report_list) = @_;
    my $report;
    my %do = ();
    my %fh = ();
    my($time, $ip_a, $port_a, $tmp, $ip_b, $port_b, $proto, $contents);
    my($host_a, $host_b);
    my($to_server, $type, $group);
    my $request;
    my(%found_crack, %found_sniff, %found_passwd, %found_passphrase);
    my $key;
    my $ref;
    my $uses_raw = 0;
    my %hostprobe = ();
    my %lowport = ();
    my %portprobe = ();
    my $interesting_host;
    my $interesting_port;

    # open all report files
    my $base_file = &main::basename($file);
    my $base_prefix = "$main::report_dir/$base_file";

    foreach $report (keys %report::info) {
	$do{$report} = 0;	#  don't do any by default
    }

    foreach $report (@report_list) {
#	print $report, "\n";
	# get file extension for this report type
	#
	die "no extension defined for report $report\n" 
	    if ! defined($report::info{$report}{'extension'});

	my $extension = $report::info{$report}{'extension'};
	
	my $report_file = "$base_prefix.$extension";

	if (! -f $report_file || $force) {
	    # if there's an open file handle for this report, oops
	    #
	    die "dup file extensions, or report listed twice: $report\n"
		if defined($fh{$report});

	    $fh{$report} = new FileHandle;
	    open($fh{$report}, ">$report_file") ||
		die "cannot open $report_file for writing ($!)\n";

	    $do{$report} = 1;

	    if ($report::info{$report}{'uses-raw'}) {
		$uses_raw = 1;
	    }
	}
    }

    # fixme: should check to see that we really need to do this first
    #
    if ($uses_raw) {

	my $command = 
	    "$main::tcpdump -q -x -n -r $file | $main::cleanup_pkts |";
	print "$command\n" if $main::debug;
	open(TCP, $command) || die "couldn't open tcpdump\n";

	$| = 1;
	while (<TCP>) {
#	    print $_;

	    if (m/icmp: echo request/) {
		($time, $ip_a, $tmp, $ip_b, $proto, $contents) = split;

		if ($do{'pingflood'}) {
		    # count number sent from/to, if > x lodge complaint?
		    # check size, if large complain about ping o death?
		    # check frequency and size, if bandwith > x?...
		}

		if ($do{'hostprobe'}) {
		    # count # different hosts pinged - 
		    # if more than HOST_PROBE_THRESH, complain?

		    foreach $tmp ($ip_a, $ip_b) {
			if (! defined($hostprobe{$tmp})) {
			    $hostprobe{$tmp} = 1;
			} else {
			    $hostprobe{$tmp}++;
			}
		    }
		}

	    } elsif (m/ (\d+\.\d+\.\d+\.\d+)\.(\d+) > (\d+\.\d+\.\d+\.\d+)\.(\d+): (tcp|udp).*\t(.*)$/) {
		$ip_a = $1; $port_a = $2; $ip_b = $3; $port_b = $4;
		$proto = $5; $contents = $6;
		
		$host_b = &host_cache::get_host_by_addr($ip_b);
		
		($to_server, $type, $group) = 
		    &services::get_tags($proto, $ip_a, $port_a, 
					$ip_b, $port_b);

		if ($do{'portprobe'} && ($port_a+0 < 1024 || $port_b+0 < 1024)) {
#		    print "portprove\n";
		    # if either port is low, this is a connection to a low port.
		    # won't work right if both are low, though one would think
		    # that that would be rare.
		    #
		    if ($port_a+0 < 1024) {
			$interesting_host = $ip_a;
			$interesting_port = $port_a;
		    } elsif ($port_b+0 < 1024) {
			$interesting_host = $ip_b;
			$interesting_port = $port_b;
		    }

		    # maintain a count of the number of unique connections to 
		    # low ports on each host
		    #
		    if (! defined($lowport{$interesting_host, $interesting_port})) {
			$portprobe{$interesting_host}++;
			$lowport{$interesting_host, $interesting_port} = 1;
#			print "probe? $interesting_host $interesting_port\n";
		    } else {
# fixme: else nothing - this is just counting packets?
#			$portprobe{$interesting_host}++;
		    }
		}
		    
		if ($to_server && $group eq 'web' && 
		    ($contents =~ m/GET (\S+) HTTP/) &&
		    ($do{'web'} || $do{'phf'})) {
		    $request = $1;

		    if ($port_b ne '80') {
			$request = "http://$host_b:$port_b$request";
		    } else {
			$request = "http://$host_b$request";
		    }

		    if ($do{'web'} && $request !~ m/\.(gif|jpg)$/) {
			$ref = &summary::make_ref($file, $proto, $ip_a, $port_a, 
						  $ip_b, $port_b);
			print { $fh{'web'} } $ref, "\t", $request, "\n";
		    }

		    # fixme - should read from config file
		    # fixme - should look for whoami, cat, xterm?

		    if ($do{'phf'}) {
			if ($request =~ m/\/phf/ || $request =~ m/\/ltk/) {
			    $ref = &summary::make_ref($file, $proto, $ip_a, 
						      $port_a, $ip_b, $port_b);
			    print {$fh{'phf'}} $ref, "\t", $request, "\n";
			}
		    }
		}

		# not quite - should cache src, dst = contents if not defined
		if ($group ne 'web') {
		    if ($do{'crack'} && 
			($contents =~ m/Guessed .*\[/ || 
			 $contents =~ m/\-\-\-\- passwords cracked as of /)) {

			$ref = &summary::make_ref($file, $proto, $ip_a, $port_a, 
						  $ip_b, $port_b);
			$found_crack{$ref} = "$proto $ip_a $port_a $ip_b $port_b";
		    }

		    if ($do{'sniff'} && 
			$contents =~ m/\-\- TCP\/IP LOG \-\-/) {
			$ref = &summary::make_ref($file, $proto, $ip_a, $port_a, 
						  $ip_b, $port_b);
			$found_sniff{$ref} = "$proto $ip_a $port_a $ip_b $port_b";
		    }

		    if ($do{'passwd'} && 
			$contents =~ m/root:[^:]*:0:0/) {
			$ref = &summary::make_ref($file, $proto, $ip_a, $port_a, 
						  $ip_b, $port_b);
			$found_passwd{$ref} = "$proto $ip_a $port_a $ip_b $port_b";
		    }

		    if ($do{'passphrase'} && 
			$contents =~ m/pass[ ]*phrase/i) {
			$ref = &summary::make_ref($file, $proto, $ip_a, $port_a, 
						  $ip_b, $port_b);
			$found_passphrase{$ref} = "$proto $ip_a $port_a $ip_b $port_b";
		    }
		}
		
		# if to ftp server
		if ($to_server && $type eq 'ftp' && $do{'ftp'}) {
		    if ($contents =~ m/(RETR|STOR|CWD) (.*)$/) {
			$host_a = &host_cache::get_host_by_addr($ip_a);

			$ref = &summary::make_ref($file, $proto, $ip_a, 
						  $port_a, $ip_b, $port_b);
			printf {$fh{'ftp'}} "%s\t%-35s %-35s %s %s\n", $ref, $host_a, $host_b, $1, $2, "\n";
		    }
		}

		# if to irc server
			
		if ($to_server && $type eq 'irc') {
		    if ($do{'dcc-send'}) {
			if ($contents =~ m/DCC CHAT/ ||
			    $contents =~ m/DCC SEND/) {
			    $ref = &summary::make_ref($file, $proto, $ip_a, 
						      $port_a, $ip_b, $port_b);
			    print {$fh{'dcc-send'}} $ref, "\t", $contents, "\n";
			}
		    }

		    if ($do{'irc-nick'}) {
			if ($contents =~ m/NICK (\S+)/) {
			    $ref = &summary::make_ref($file, $proto, $ip_a, 
						      $port_a, $ip_b, $port_b);
			    print {$fh{'irc-nick'}} $ref, "\tNICK $1\n";
			}
		    }
		}
	    }
	}
    }
#    print "\n";

    close(TCP);

    # deal with things we collapsed - crack, sniff, passwd files
    #
    if ($do{'crack'}) {
	foreach $key (keys %found_crack) {
	    print {$fh{'crack'}} $key, "\t", $found_crack{$key}, "\n";
	}
    }

    if ($do{'sniff'}) {
	foreach $key (keys %found_sniff) {
	    print {$fh{'sniff'}} $key, "\t", $found_sniff{$key}, "\n";
	}
    }

    if ($do{'passwd'}) {
	foreach $key (keys %found_passwd) {
	    print {$fh{'passwd'}} $key, "\t", $found_passwd{$key}, "\n";
	}
    }

    if ($do{'passphrase'}) {
	foreach $key (keys %found_passphrase) {
	    print {$fh{'passphrase'}} $key, "\t", $found_passphrase{$key}, 
	    "\n";
	}
    }

    if ($do{'hostprobe'}) {
	if (keys %hostprobe > 10) {
	    foreach $tmp (keys %hostprobe) {
		printf {$fh{'hostprobe'}} "\t%5d %s\n", $hostprobe{$tmp}, $tmp;
	    }
	}
    }

    if ($do{'portprobe'}) {
	foreach $tmp (keys %portprobe) {
#	    print "hmmm...$tmp $portprobe{$tmp}\n";
	    if ($portprobe{$tmp} > 10) {
		printf {$fh{'portprobe'}} "\t%5d %s\n", $portprobe{$tmp}, $tmp;
	    }
	}
    }

    # should deal here with searches that require packet reassembly
    # or more advanced processing:
    #
    #	search client->server telnet stream with ^H and ^? processed
    #
    #   search server->client telnet or rlogin traffic with escape codes handled?

    # also: reports based on summaries
		    
    # close all reports
    #
    foreach $report (@report_list) {
	close($fh{$report}) if $do{$report};
    }
}

1;
