package summary_window;

use strict;

##############################################################################
# summary_window package - create a scrollable window for viewing the
# summary of a tcpdump log.
#
# exports &create
#
# copyright (c) 1997 by Steve Romig and The Ohio State University
#
##############################################################################
# $Id: summary_window.pm,v 1.1 1997/05/27 21:03:47 romig Exp $
##############################################################################
# $Log: summary_window.pm,v $
# Revision 1.1  1997/05/27 21:03:47  romig
# Initial revision
#

#
# called when a line from the display is selected - parse out the src and 
# dst ip and port and call &show_traffic to browse that traffic.
#

sub select_traffic {
    my($ref) = @_;

    print "select traffic: $ref\n";
    my($file, $line) = split(/\|/, $ref);

    # fixme: dunno what this will do if aimed at a icmp or arp line
    my($tmp, $tmp, $ip_a, $port_a, $ip_b, $port_b, $proto, $tmp) = 
	&summary::parse($line);

    &session_window::create($file, $proto, $ip_a, $port_a, $ip_b, $port_b);
}

#
# fill the text box with a summary of the named session, after applying 
# suitable include/exclude filtering.
#
sub redraw_summary {
    my($box, $file, $include, $exclude) = @_;

    my $e_include = '';
    my $e_exclude = '';
    my $command = '';
    my @result;
    my %seen;
    my $line;

    if ($include ne '') {
	$e_include = "$main::egrep '$include' |";
    }

    if ($exclude ne '') {
	$e_exclude = "$main::egrep -v '$exclude' |";
    }

    @result = &summary::get($file, $e_include, $e_exclude);

    # lord, bless this box.  make it editable, delete its contents...
    #
    bless $box, "Tk::Text";
    $box->configure(-state => "normal");
    $box->delete('1.0', 'end');

    # insert the lines from the summary into the box
    #
    my $maxlen = 0;
    my $i;
    my @traffic_list;

    # eat the newline on the last 
    #
    chomp($result[$#result]) if $#result > -1;

    foreach ($i = 0; $i <= $#result; $i++) {
	$maxlen = length $result[$i] if ($maxlen < length $result[$i]);
	$box->insert('end', $result[$i], ['go', "go-$file|$result[$i]"]);
    }

    my $height = 4;
    my $width = 90;

    $height = &main::min(30, scalar @result) unless(scalar @result < 4);
    $width = &main::min(90, $maxlen);

    $box->configure(-state	=> "disabled",
		    -width	=> $width,
		    -height	=> $height);
    @result = ();
}

#
# edit a log file.  scary stuff.  this was added *specifically* and *only* 
# to allow us to remove "extra stuff" from the logs.  i don't plan on 
# extending this to allow other forms of log file editing.  
#
# if file.original does not exist, copy this to it.  if it does, don't
# bother with a backup.
#
sub delete {
    my($file, @lines) = @_;

    my $result;

    # create a copy of this if $file.original does not exist
    #
    my $original = $file . "---original";
    if (! -f $original) {
	$result = system("$main::cp $file $original");
	if ($result/256 != 0) {
	    warn "copy of $file to $original failed\n";
	    unlink $original;
	}
    }

    # if the original does not exist, then give up
    #
    return if ! -f $original;

    # delete $file---edit if it exists
    #
    my $edit = $file . "---edit";
    unlink $edit if -f $edit;

    # for the edit, we're going to tcpdump from $file (current version) 
    # to $edit.  if that seemed to work, then we'll unlink $file and move
    # $edit into its place...
    #
    my($match, $command, $line, $tmp, $ip_a, $port_a, $ip_b, $port_b, 
       $proto, $rest);

    foreach $line (@lines) {
	($tmp, $tmp, $ip_a, $port_a, $ip_b, $port_b, $proto, $rest) = 
	    &summary::parse($line);

	$match = "not \( \( src host $ip_a and src port $port_a and dst host $ip_b and dst port $port_b and $proto \) or \( dst host $ip_a and dst port $port_a and src host $ip_b and src port $port_b and $proto \) \)";
	$command = "$main::tcpdump -r $file -w $edit $match";
	print "$command\n";

	$result = system split(' ', $command);

	if ($result/256 == 0) {
	    unlink $file;
	    if (! rename $edit, $file) {
		warn "could not rename $edit to $file ($!)\n";
		warn "giving up delete - check things manually!\n";
		return;
	    }
	} else {
	    warn "tcpdump failed, giving up - check things manually\n";
	    return;
	}
    }
}

#
# create the log summary window for a specific log file
#
# this scrollable window contains the summary of the sessions from a log.
# 
# double click on a session brings up the session window for that entry.
#

sub create {
#    shift;			# throw away 
    my($file) = @_;		# file name was passed as arg

    my $include = ' tcp';
    my $exclude = ' web| junk';

    # create window
    
    my $window = $log_window::main->Toplevel;

    $window->title("Summary of $file");

    # create the text widget now, i think i need it for the references
    #
    my $text = $window->Text(-relief => 'sunken',
			     -background => $main::background_color,
			     -foreground => $main::foreground_color,
			     -width => 90,
			     -height => 30,
			     -setgrid => 'yes');

    &window::define_tags($text, \&summary_window::select_traffic);

    # menu bar
    #
    # +------------------------------
    # | Dismiss Redraw View Reports Delete
    # |
    #
    my $menu = $window->Frame(-relief => 'raised', -bd => 2);
    $menu->pack(-side => 'top', -fill => 'x');

      my $dismiss_button = 
	  $menu->Button(-text => "Dismiss",
			-command => ['destroy', $window]);;
      $dismiss_button->pack(-side => 'left');

      my $redraw_button = 
	  $menu->Button(-text => "Redraw",
			-command => 
			    sub {&redraw_summary($text,
					       $file,
					       $include,
					       $exclude)});
      $redraw_button->pack(-side => 'left');

      my $viewmenu = $menu->Menubutton(-text => 'View', -underline => 0);

      $viewmenu->command(-label   => 'All traffic',
                         -command => sub {
			     $include = ''; 
			     $exclude = '';
                             &redraw_summary($text,
					   $file,
					   $include,
					   $exclude)});

      $viewmenu->command(-label   => 'TCP, no web, no junk',
                         -command => sub {
			     $include = 'tcp';
			     $exclude = ' web| junk';
                             &redraw_summary($text,
					   $file,
					   $include,
					   $exclude)});

      $viewmenu->command(-label   => 'login, ftp and irc',
			 -command => sub {
			     $include = ' login| ftp| irc'; 
			     $exclude = '';
			     &redraw_summary($text,
					   $file,
					   $include,
					   $exclude)});

      $viewmenu->command(-label   => 'mail',
			 -command => sub {
			     $include = 'mail'; 
			     $exclude = '';
			     &redraw_summary($text,
					   $file,
					   $include,
					   $exclude)});

      $viewmenu->command(-label   => 'web and gopher',
			 -command => sub {
			     $include = ' web| gopher'; 
			     $exclude = '';
			     &redraw_summary($text,
					   $file,
					   $include,
					   $exclude)});

      $viewmenu->pack(-side=>'left');

      my $reports_button = 
	  $menu->Button(-text => "Reports",
			-command => 
			    sub {&report_window::create('all', $file)});
      $reports_button->pack(-side => 'left');

      my $delete_menu = $menu->Menubutton(-text => 'Delete_', -underline => 0);
      $delete_menu->command(-label   => 'Delete selected entries',
                         -command => sub {
                             my $line = $text->get('sel.first', 'sel.last');
                             my @lines = split '\n', $line;
			     &delete($file, @lines);

			     # delete the summary so it gets remade
			     &summary::delete($file);
			     &redraw_summary($text,
					   $file,
					   $include,
					   $exclude);
			 });

      $delete_menu->pack(-side=>'left');

    # include/exclude entry boxes
    #
    # +------------------------------
    # | Dismiss Redraw View Delete
    # | include....     exclude.....
    #
    my $inc_exc_frame = $window->Frame;

      # include box stuff...
      #
      my $include_frame = $inc_exc_frame->Frame;

      my $label_include = $include_frame->Label(-text => 'Include');
      my $entry_include = $include_frame->Entry(-width => 40,
                                                -background => 
						    $main::background_color,
                                                -foreground => 
						    $main::foreground_color,
                                                -textvariable => \$include);

      $label_include->pack(-side => 'left');
      $entry_include->pack(-side => 'left');

      $include_frame->pack(-side => 'left');

      # exclude box stuff...
      #
      my $exclude_frame = $inc_exc_frame->Frame;

      my $label_exclude = $exclude_frame->Label(-text => 'Exclude');
      my $entry_exclude = $exclude_frame->Entry(-width => 40,
                                                -background => 
						    $main::background_color,
                                                -foreground => 
						    $main::foreground_color,
					        -textvariable => \$exclude);

      $label_exclude->pack(-side => 'left');
      $entry_exclude->pack(-side => 'left');

      $exclude_frame->pack(-side => 'right');

    $inc_exc_frame->pack;

    #field description for the text box
    #
    # +------------------------------
    # | Dismiss Redraw View Delete
    # | include....      exclude....
    # | description
    #
    my $description = $window->Text(-relief => 'flat',
                                    -width => 80,
                                    -height => 1);
    $description->insert('0.0', ' # Packets  Time                      Src IP_Port            Dst IP_Port Protocol');
    $description->pack(-side => 'top',
                       -fill => 'x',
                       -expand => 'y');
    $description->configure(-state => 'disabled');

    # fill in the listbox
    #
    # +------------------------------
    # | Dismiss Redraw View Delete
    # | include....      exclude....
    # | description
    # ++-----------------------
    # || summary text here
    # ||
    #
    &redraw_summary($text, 
                  $file, 
		  $include, 
	          $exclude);

    # create the scroll bar
    #
    # +------------------------------
    # | Dismiss Redraw View Delete
    # | include....      exclude....
    # | description
    # + +-----------------------+ S
    # | | summary text here     | C
    # | |                       | R
    # | |                       | O
    # | |                       | L
    # | +-----------------------+ L
    #
    my $scroll = $window->Scrollbar(-command => ['yview', $text]);

    $text->configure(-yscrollcommand => ['set', $scroll]);
    $text->pack(-side => 'left',
                -fill => 'both',
		-expand => 'yes');

    $scroll->pack(-side => 'right',
		  -fill => 'y');
}

1;
