#include <stdio.h>
#include <X11/Xlib.h>

#define WIDTH 640
#define HEIGHT 800

Display *display;
Window dw;
Pixmap pm;
GC gc, cleargc;
int Xwidth, Xheight;
int needspause = 0;

ExtraPause()

{
#define BUFSIZE 32

    char buffer[BUFSIZE];
    XEvent		    xevent;

    XFlush(display);

    printf("pause\n");

    while (1) {
	XNextEvent(display, &xevent);
 
	if (xevent.type == Expose) {
	    printf("  ...expose\n");
	    ExtraExpose();
	} else if (xevent.type == KeyPress) {
	    printf("  ...keypress\n");
	    return;
	} else if (xevent.type == ButtonPress) {
	    exit(0);
        }
    }

    needspause = 0;
}

ExtraExpose() 

{
    XCopyArea(display, pm, dw, gc, 0, 0, Xwidth, Xheight, 0, 0);
}

ExtraInit() 

{
  char *dispname;
  int screen;
  unsigned long valuemask, white, black, depth;
  XSetWindowAttributes attr;
  XEvent event;
  XGCValues gcval;
  Font font;

  extern char *getenv();

  Xwidth = WIDTH;
  Xheight = HEIGHT;

  dispname=getenv("DISPLAY");
  if (dispname==NULL) {
    fprintf (stderr,"No display set!\n");
    exit(1);
  }

  /* Connect to the X server */
  display=XOpenDisplay(dispname);
  if (display==NULL) {
    fprintf (stderr,"Can't open display!\n");
    exit(1);
  }

  /* look up some stuff to be used later */
  screen=DefaultScreen(display);
  white=WhitePixel(display, screen);
  black=BlackPixel(display, screen);
  depth=DefaultDepth(display, screen);
  font=XLoadFont(display,"fixed");

  /* Prepare to create a window to draw in */
  valuemask=0;
  attr.background_pixel=white;
    valuemask |= CWBackPixel;
  attr.backing_store=NotUseful;		valuemask |= CWBackingStore;
  attr.cursor=None;			valuemask |= CWCursor;
  attr.event_mask=KeyPressMask|ExposureMask;
    valuemask |= CWEventMask;

  /* Create the window */
  dw = XCreateWindow(display,RootWindow(display,DefaultScreen(display)),
		100,100,Xwidth,Xheight,0,depth,
		InputOutput,(Visual *)CopyFromParent, valuemask,&attr);

  /* Create an offscreen pixmap to draw in so we can handle expose events. */
  /* Things will be drawn to this pixmap, which will be copied to the */
  /* window */
  pm = XCreatePixmap(display,dw,Xwidth,Xheight,depth);

  /* Create Graphics Contexts */
  /* One for drawing... */
  valuemask=0;
  gcval.foreground=black;		valuemask |= GCForeground;
  gcval.background=white;		valuemask |= GCBackground;
  gcval.function=GXcopy;		valuemask |= GCFunction;
  gcval.graphics_exposures=False;	valuemask |= GCGraphicsExposures;
  gcval.font=font;                      valuemask |= GCFont;
  gc=XCreateGC(display,dw,valuemask,&gcval);

  /* ...and one (almost the same) for erasing */
  gcval.foreground=white;
  gcval.background=white;
  cleargc=XCreateGC(display,dw,valuemask,&gcval);

  XFillRectangle(display, pm, cleargc, 0, 0, Xwidth, Xheight);
  XCopyArea(display, pm, dw, gc, 0, 0, Xwidth, Xheight, 0, 0);

  /* put the window on the screen */
  XMapWindow(display,dw);
}

#ifdef FOO
------------------------------------------------------------------------------
  while(XNextEvent(display,&event)) {
    printf("hey there, hi there, ho there\n");
    printf("type=%d\n",event.type);

    if (event.type == NoExpose) {
      /* these were generated when graphics_exposures was set to True */
      printf("nothing\n");
    } else if (event.type == KeyPress) {


      XDrawString(display, pm, gc, random()%WIDTH, random()%HEIGHT,
		"Hello!", 6);
      XCopyArea(display, pm, dw, gc, 0, 0, WIDTH, HEIGHT, 0, 0);

    } else if (event.type == Expose) {
      XCopyArea(display, pm, dw, gc, 0, 0, WIDTH, HEIGHT, 0, 0);
    }
  }
  
  XCloseDisplay(display);
}
#endif
